/*
   ----------------------------------------------------------------------------
   ReportFromArray.prg
   Sample code from The dBASE Reports Book, second edition
   Chapter 21
   Copyright  2016, Kenneth J. Mayer
   
   Program used to generate a report from an array. This is mostly an exercise
   to show how to build a report and the various objects used on a report
   "on-the-fly", but it shows as well the basic ideas of using an array
   for a report.
   
   This may be used for other purposes, as needed. 
   ----------------------------------------------------------------------------
*/
close databases

// Set up the database object:
dData = new database()
dData.databaseName := "DBASEREPORTS"
dData.active       := true

// For this to work we need a simple table with
// one record. This will be connected to the
// streamSource of the report once we build it
// If it exists, delete it:
if file( "TempTable.dbf" )
   erase TempTable.dbf
endif
// This table will be created in the current directory
cCmd = "create table TempTable (CharField char(20))"
&cCmd.
// the table needs a single row so we have one detailBand
// in our report:
use TempTable
append blank
replace CharField with "Some Data"
use

// We need to open the dogs table, pull some data
// from it and load it into an array:
qDogs = new query()
qDogs.database := dData
qDogs.sql      := "select * from dogs"
qDogs.active   := true
qDogs.rowset.indexName := "Name"

// Next we need an array:
aExample = new array( 1,2 ) // start with one row and two columns

// loop through the table and get nMaxRecords records
// for the purpose of our test:
nMaxRecords = 30
nCurrent    = 0

// start at top of table
qDogs.rowset.first()

// make sure we get nMaxRecords and we don't
// go past the endOfSet for the rowset:
do while nCurrent < nMaxRecords and not qDogs.rowset.endOfSet
   nCurrent++ // increment counter
   
   // if more than one row, grow the array by one row:
   if nCurrent > 1
      aExample.grow( 1 )
   endif
   // fill in the data
   aExample[nCurrent, 1] = qDogs.rowset.fields["Name"].value.rightTrim()
   aExample[nCurrent, 2] = qDogs.rowset.fields["Born"].value
   // next record:
   qDogs.rowset.next()
enddo // end of loop

// now we have the array, we can close the query:
qDogs.active := false
release object qDogs
qDogs := null
// don't need the database object, either:
dData.active := false
release object dData
dData := null

// we will need a reference to the temporary table:
qTemp = new query()
qTemp.sql    := "select * from TempTable"
qTemp.active := true

// here is where it gets tricky, we're going to create the 
// report objects completely in code. To assist I used 
// an existing report for the top/left and various other
// properties of some of the controls:
oRep = new report()
oRep.metric := 3 // Inches
//oRep.mdi    := false // we want the preview Window to be modal

// objects on the report:
oPT = new pageTemplate( oRep ) // contained by the report
// set the pageTemplate's margins and size:
oPT.height       := 11.0
oPT.width        := 8.5
oPT.marginTop    := 0.5
oPT.marginLeft   := 0.5
oPT.marginBottom := 0.5
oPT.marginRight  := 0.5

// streamFrame:
oSF = new streamFrame( oPT ) // contained by the pageTemplate
oSF.height = 8.65
oSF.left = 0.0
oSF.top = 1.0
oSF.width = 7.5

// streamSource:
oSS = new streamSource( oRep ) // contained by the report

// text on the pageTemplate for report headings:
oPT.MainTitle = new text( oPT )
oPT.MainTitle.text            := "My Company Name"
oPT.MainTitle.metric          := 3	// Inches
oPT.MainTitle.height          := 0.3646
oPT.MainTitle.width           := 3.0833
oPT.MainTitle.alignHorizontal := 1	// Center
oPT.MainTitle.fontSize        := 20.0
oPT.MainTitle.fontItalic      := true
oPT.MainTitle.left            := 2.2083
oPT.MainTitle.top             := 0.0

// secondary title:
oPT.SecondaryTitle = new text( oPT )
oPT.SecondaryTitle.text            := "Report From Array"
oPT.SecondaryTitle.height          := 0.2708
oPT.SecondaryTitle.width           := 2.3333
oPT.SecondaryTitle.metric          := 3	// Inches
oPT.SecondaryTitle.alignHorizontal := 1	// Center
oPT.SecondaryTitle.fontSize        := 16.0
oPT.SecondaryTitle.left            := 2.5833
oPT.SecondaryTitle.top             := 0.3854

// date:
oPT.DateText = new text( oPT )
oPT.DateText.text            := {||cmonth(date())+" "+day(date())+", "+year(date())}
oPT.DateText.height          := 0.2083
oPT.DateText.width           := 2.1042
oPT.DateText.metric          := 3	// Inches
oPT.DateText.alignHorizontal := 1	// Center
oPT.DateText.fontSize        := 12.0
oPT.DateText.left            := 2.6979
oPT.DateText.top             := 0.6771

// assignments necessary for report to function:
oRep.firstPageTemplate        := oPT
oPT.nextPageTemplate          := oPT
oSF.streamSource              := oSS
oSS.rowset                    := qTemp.rowset     // now we have a detailBand

// at this point we have one detailBand and everything else
// will go on that:
oDT = oSS.detailBand
// start with some column heading text:
oDT.NameHeading = new text( oDT )
oDT.NameHeading.text            := "Name"
oDT.NameHeading.fontBold        := true
oDT.NameHeading.fontName        := "Arial"
oDT.NameHeading.fontSize        := 10
oDT.NameHeading.suppressIfBlank := true
oDT.NameHeading.canRender       := {||this.parent.firstOnFrame}
oDT.NameHeading.metric          := 3 // Inches
oDT.NameHeading.height          := 0.2083
oDT.NameHeading.left            := 0.0208
oDT.NameHeading.top             := 0
oDT.NameHeading.width           := 0.5729

// start with some column heading text:
oDT.BornHeading = new text( oDT )
oDT.BornHeading.text            := "Born"
oDT.BornHeading.fontBold        := true
oDT.BornHeading.fontName        := "Arial"
oDT.BornHeading.fontSize        := 10
oDT.BornHeading.suppressIfBlank := true
oDT.BornHeading.canRender       := {||this.parent.firstOnFrame}
oDT.BornHeading.metric          := 3 // Inches
oDT.BornHeading.height          := 0.2083
oDT.BornHeading.left            := 3.9896
oDT.BornHeading.top             := 0
oDT.BornHeading.width           := 0.6979

// now we build the text controls for the individual rows
// of the array. We will rely heavily on macros here as it
// is the easiest way to do this in a loop:
for i = 1 to aExample.size / 2 // two columns

    // name of the text control (Name field)
    cName = "oDT.text"+i+"_1" // first row would name this text1_1
    // create the text control
    cCmd = cName + " = new text( oDT )"
    &cCmd. // execute macro
    
    // set properties (note double quotes and brackets for delimiters):
    cCmd = cName+".text = ["+aExample[ i, 1 ]+"]"
    &cCmd.
    cCmd = cName+".width = 3.6979"
    &cCmd.
    cCmd = cName+".top = "+(i*.21) // 0.21" tall
    &cCmd.
    
    // repeat for second field (Born)
    cName = "oDT.text"+i+"_2" // first row would name this text1_2
    // create the text control
    cCmd = cName + " = new text( oDT )"
    &cCmd. // execute macro
    
    // set properties (note single and double quotes for delimiters):
    cCmd = cName+".text = '"+aExample[ i, 2 ]+"'"
    &cCmd.
    cCmd = cName+".width = 0.9479"
    &cCmd.
    cCmd = cName+".top = "+(i*.21) // 0.21" tall
    &cCmd.
    cCmd = cName+".left = 4.0"
    &cCmd.
    
next // element in aExample array

// if all is well, we need to render the report:
oRep.render()

// cleanup:
qTemp.active := false
release object qTemp
qTemp := null

// delete the temporary table:
erase TempTable.dbf
