// Metric: Non Specific
/*
    GridColumnIndexChange.cc
    Programmer ...: Ken Mayer
    Date..........: February 13, 2004
    Notes.........: A custom grid that can be used to allow the
                    user to click on a columnHeading and 
                    change the sort sequence of a rowset.
                    Note that this is not perfect, it only
                    works with grids that are datalinked
                    to a single rowset, and there are probably
                    some other issues that might cause problems.
                    Cobbled together based on a request in the
                    dBASE Wishlist newsgroup by Pier Alberto
                    Guidotti.

                    A note of caution, changing properties
                    of the grid slows things down ... and 
                    focus moves to the wrong column. If you
                    change both the column color and the
                    heading properties, this means two iterations
                    through the grid, which will slow things
                    down even more.

    Usage Notes...: Make sure this is available as a custom
                    component:
                        set procedure to :dUFLP:GridColumnIndexChange.cc additive
                    Drag to a form.
                    Set the datalink.
                    Set the columns array (it won't work without it,
                        and currently we're not testing for that).
                    Run the form ... 

                    ** See below for using the properties as noted
                       in the next section ...

    Properties...: warnMessage -- this property will
                       cause a message to display on screen
                       for the user if the column heading selected
                       is for a column that doesn't have an index tag
                       by the fieldname ... default: false
                   ----------------
                   changeColumn -- this property will affect the
                       display of the columns, by changing
                       properties as noted below. Default is true.
                   colorNormal -- this is the colorNormal
                       for the grid, it defaults to the standard
                       windowtext/window colors.
                   colorHighlight -- this is the colorHighlight
                       for the grid, standard colors again for
                       the default.
                   indexColumnColor -- not standard, this
                       defaults to "white/lightgrey", and is used
                       to show the column that has the current index.
                       You can change the default color in the
                       form's onOpen event or an overridden 
                       Open/ReadModal method (see below).
                    ----------------
                    changeHeading -- this property will
                       affect the display of the columnHeadings,
                       by changing properties as noted below. 
                       Default is false.
                    indexHeadingColor -- not standard, this
                       defaults to "white/lightgrey", and
                       is used to show the column that has the
                       current index.
                    indexHeadingBold -- this property can be
                       used to change the heading to/from
                       bold if it is the column heading that
                       is used for the index. Default is true.
                    indexHeadingItalic -- this property can be
                       used to change the heading to/from
                       Italics if it is the column heading that
                       is used for the index. Default is false.
                    indexHeadingUnderline -- this property can be
                       used to change the heading to/from
                       Underline if it is the column heading that
                       is used for the index. Default is false.


                    ----------------
                    You may want to set the grid's IndexColumnColor
                    color (and other properties) to whichever 
                    column is the one that the rowset defaults 
                    to ... see below:

                    These options should probably be set in a form
                    onOpen event or overridden Open or ReadModal
                    method. Something like:

                    function form_onOpen
                       // only use if you want the warning messages:
                       form.GridColumnIndexChange1.WarnMessage := true
                       // Set the color you want the column to display
                       // when it is the one that the data is sorted on:
                       form.GridColumnIndexChange1.IndexColumnColor := "yellow/red"
                       // This sets the colorNormal for the column that
                       // currently has the index set:
                       form.GridColumnIndexChange1.columns[ 2 ].editorControl.colorNormal := ;
                            form.GridColumnIndexChange1.IndexColumnColor
                    return
*/
class GridColumnIndexChange( oParent ) of grid( oParent ) custom

   // call code when the grid opens:
   this.onOpen               := class::SetColumnHeadings
   // special properties -- flags:
   this.warnMessage           = false
   this.changeColumn          = true
   this.changeHeading         = false
   // colors -- the first two are standard properties
   // of the grid:
   this.colorNormal          := "windowtext/window"
   this.colorHighlight       := "highlightText/highlight"
   // special:
   this.indexColumnColor      = "white/lightgrey"
   this.indexHeadingColor     = "white/lightgrey"
   this.indexHeadingBold      = true
   this.indexHeadingItalic    = false
   this.indexHeadingUnderline = false

   function SetColumnHeadings
      // this should fire in the grid's onOpen, and set the
      // column headings' onLeftMouseUp to fire the code
      // 'ChangeIndex':
      private i
      // Check to see if we have a columns array:
      if this.columns.size == 0 // no column set
         if this.WarnMessage
            msgbox( "Columns array not set for this grid!", ;
                    "Warning", 48 )
         endif
         return
      endif

      // loop through each column:
      for i = 1 to this.columnCount
          this.columns[ i ].headingControl.onLeftMouseUp = class::ChangeIndex
      next
   return

   function ChangeIndex(flags, col, row)
      // The grid's SetColumnHeading code, which is called
      // from the grid's onOpen event, sets the headingControl's
      // onLeftMouseUp event to fire this routine. This
      // attempts to set an index for any field that has
      // one by the fieldname of the column ...
      private nColumn, aColumns, cField, oRowSet, cOldIndex, i
      nColumn = this.parent.currentColumn  // get current column
      aColumns = this.parent.getColumnOrder() // returns an array
      cField = aColumns[ nColumn, 2 ] // fieldName
      // Now we have the fieldname, try to set the 
      // index ... we're going to try just based on the
      // fieldName ... there's no good way (without using
      // a tableDef object) to find out if there's
      // an index tag by a different name that will
      // do the trick. 
      oRowSet = this.parent.dataLink
      // in case it didn't work:
      cOldIndex = oRowset.indexName
      try
         oRowset.indexName := cField
         // don't update the grid until we're done
         oRowset.notifyControls := false

         // if this property is true, then set/reset
         // colors:
         if this.parent.changeColumn
            // change the color of the grid
            // to the standard colorNormal:
            for i = 1 to this.parent.columnCount
                this.parent.columns[ i ].editorControl.colorNormal :=;
                     this.parent.colorNormal
            next
            // change the color of the column we're changing
            // the index to to the specially defined color:
            this.parent.columns[ nColumn ].editorControl.colorNormal := ;
                     this.parent.IndexColumnColor
         endif
         // if this property is true, then set/reset
         // heading properties (note, we have to do this
         // in two loops, because the user might want
         // only the heading to change ... so we can't 
         // subsume this into the loop above):
         if this.parent.changeHeading
            // change the color of the grid
            // to the standard colorNormal:
            for i = 1 to this.parent.columnCount
                   this.parent.columns[ i ].headingControl.colorNormal :=;
                        this.parent.headingColorNormal
                   this.parent.columns[ i ].headingControl.fontBold :=;
                        this.parent.headingFontBold
                   this.parent.columns[ i ].headingControl.fontItalic :=;
                        this.parent.headingFontItalic
                   this.parent.columns[ i ].headingControl.fontUnderline :=;
                        this.parent.headingFontUnderline
            next
            // change the color/font properties of the column heading
            // we're changing the index to:
            this.parent.columns[ nColumn ].headingControl.colorNormal := ;
                     this.parent.indexHeadingColor
            this.parent.columns[ nColumn ].headingControl.fontBold := ;
                     this.parent.indexHeadingBold
            this.parent.columns[ nColumn ].headingControl.fontItalic := ;
                     this.parent.indexHeadingItalic
            this.parent.columns[ nColumn ].headingControl.fontUnderline := ;
                     this.parent.indexHeadingUnderline
         endif
         // Tab to the column selected?
         this.parent.setFocus()
         if nColumn > 1
            // backtab to the first column (just to be sure 
            // of where we are in the grid):
            keyboard replicate( "{Shift+Tab}", this.parent.currentColumn-1 )
            // tab to the current column:
            keyboard replicate( "{Tab}", nColumn-1 )
         endif
         // allow updates to grid now:
         oRowset.notifyControls := true
         // ensure we update the grid:
         oRowset.refreshControls()
      catch( Exception E )
         if e.code # 239 // "Index does not exist"
            msgbox( "Unexpected error: "+e.code+" - "+e.message,"Error!", 16 )
         else
            // optional message:
            if this.parent.warnMessage
               msgbox( "No index is set for this field ...", "Warning", 48 )
            endif
         endif
         // error, set index to what it was before ...
         oRowset.indexName := cOldIndex
      endtry
   return

endclass