/*
   CascadeUpdateExample.prg
   A sample dBL program from 
   The dBASE Book
   by Kenneth J. Mayer. 
   Program copyright 2012. 
   Permission granted for use as a learning tool for dBASE Plus.

   Sample from: Chapter 8 
   This code will show how Cascade Deletes and Cascade Updates
   can be implemented in your application. As mentioned in the 
   book, Cascade Updates are not generally a good idea, as if
   handled improperly you can generate orphaned child records.
*/

clear

// As this code relies on the data being exactly as needed, 
// we're not asking about tables existing ... running this
// routine once will delete data in two of the tables. This
// will ensure the data is how it shoudl be at the beginning:
do MakeChapter8Tables

// -----------------------------------------------------
// set up queries

// Parent table:
qParent = new query()
qParent.sql := "select * from ParentTest"
qParent.active := true
// short references:
rParent = qParent.rowset
fParent = rParent.fields
// set an index:
rParent.indexName := "LinkField"

// Child table:
qChild = new query()
qChild.sql := "select * from ChildTest"
qChild.active := true
// short references:
rChild = qChild.rowset
fChild = rChild.fields
// set the index:
rChild.indexName := "LinkField"

// Set event handlers:
rParent.canEdit = rowset_canEdit
rParent.canSave = rowset_canSave

// set the masterRowset/masterFields:
rChild.masterRowset := rParent
rChild.masterFields := "LINKFIELD"

// display information in tables:
? "Data in tables prior to test:"
?
rParent.first()
do while not rParent.endOfSet
   ? fParent["LinkField"].value+" -- "+fParent["ParentData"].value

   // child table:
   rChild.first()
   do while not rChild.endOfSet
      ? "   "+fChild["LinkField"].value+" -- "+fChild["ChildData"].value
      rChild.next()
   enddo

   // navigate in parent table:
   rParent.next()
enddo

?
wait "Press a key to continue test ..."

// to update a row in the parent table, we 
// need to navigate to the record, then
// try changing the value in the primary key ... 
rParent.first()

// the first parent record has three children records, 
// so we'll try that one:
?
? "Changing value in primary key field of parent table."
?
// start edit mode, and call 
// the rowset's canEdit event handler
// below:
rParent.beginEdit()
// change the primary key value:
fParent["LinkField"].value := 10
// call the canSave event handler:
rParent.save()

// display information in tables:
? "Data in tables after edit of first row"
?
rParent.first()
do while not rParent.endOfSet
   ? fParent["LinkField"].value+" -- "+fParent["ParentData"].value

   // child table:
   rChild.first()
   do while not rChild.endOfSet
      ? "   "+fChild["LinkField"].value+" -- "+fChild["ChildData"].value
      rChild.next()
   enddo

   // navigate in parent table:
   rParent.next()
enddo

// --------------------------------------------------------------------
// Cleanup
// --------------------------------------------------------------------
qParent.active := false
release object qParent
qParent := null

qChild.active := false
release object qChild
qChild := null

// end of program

// --------------------------------------------------------------------
// Event Handlers needed to make this work:
// --------------------------------------------------------------------

function rowset_canEdit
   // Save the value of the primary key to a custom
   // property of th erowset:
   this.PrimaryKey = this.fields["LINKFIELD"].value
return true

function rowset_canSave
   local r, nNewKey
   // we need to see if the fields modified include the 
   // primary key -- we saved the original value
   // of the field in a custom property of the 
   // rowset object:
   if this.fields["LinkField"].value # this.PrimaryKey

      // short reference to rowset of child table:
      r = rChild

      // we have to sever the relationship between the
      // tables (?)
      r.masterRowset := null
      r.masterFields := null

      // loop through the fields in the child table,
      // by first setting a filter so we are only modifying
      // the ones we want:
      nNewKey = this.fields["LinkField"].value
      r.filter = [LinkField=]+this.PrimaryKey

      // first row of child table
      r.first()
      // loop and make change:
      do while not r.endOfSet
         // change to new key
         r.fields["LinkField"].value := nNewKey
         // save
         r.save()
         // navigate to first record in filter
         // otherwise we may be at "end of set"
         r.first()
      enddo

      // clear the filter:
      r.filter := null

      // re-set the masterRowset/masterFields:
      r.masterRowset := this
      r.masterFields := "LINKFIELD"

   endif // primary key has changed

   // any other code you may wish to use ...

return true

// -- end of file

