// -- Metric: Not an issue -- uses default size

/*-----------------------------------------------------------------------------

		Class name:	extCombobox
   		 Author: Gary White
	Last modified: 9 May 2000

   Description:

         This class performs an incremental search of the list items
         and uses "auto-complete" to display the item found and highlight
         the remainder of the text after the characters actually typed.

         If the datasource is a field and the text entered is not 
         found, the user will optionally be prompted to see if they
         want the new item added to the table.

         NOTE:  Virtually none of this works if you set the style
         		 property to 2 (DropDownList).


      Extends the built-in Combobox by adding the following:
-------------------------------------------------------------------------------
		Added Properties:

      	AddPrompt		Logical		Determines whether the user will be
         									prompted to add an entry. If true and 
                                    datasource is a field and the user 
                                    enters a value not on the list, they 
                                    will be prompted to see if they want the
                                    entry added to the list.  If false, there
                                    will be no prompt, nor will the value be
                                    added.  If RestrictToList is TRUE, this
                                    property is ignored.

         BeepOnError		Logical		If RestrictToList is true, determines
         									if a beep will sound if the user types
                              		an entry not in the list

   											Legal values for BeepOnError include:

                                    These are #define'd below:
                           			NO_SOUND							 0
                                    PC_SPEAKER						 1
                                    DEFAULT_SOUND  				 0xFFFFFFFF

                                    These defined in Winuser.h
                                    MB_ICONHAND                 0x00000010
                                    MB_ICONQUESTION             0x00000020
                                    MB_ICONEXCLAMATION          0x00000030
                                    MB_ICONASTERISK             0x00000040

                                    Value of NO_SOUND means there is no audible
                                    indication of an illegally typed value

                                    Value of PC_SPEAKER sounds the bell on the
                                    PC's speaker

                                    All other legal values play the sounds 
                                    selected by the user in the control panel
                                    through the PC's sound card.

                                    If an illegal value is assigned to the 
                                    BeepOnError property, no sound will be 
                                    played.

         RestrictToList	Logical		Determines if the user can type an
         									entry not in the list
-------------------------------------------------------------------------------
      Added methods:

     		DropDown(bShow)				Displays, or hides the drop down list
         									depending on the value of bShow

         FindString( cString)			Searches the list and returns TRUE if
                           			cString is found, or FALSE if not

         SelectedText					Returns the text selected (highlighted)
         									by the user

         SelEnd							Returns the character position of the 
         									last selected character

         SelLength						Returns the length of the current 
         									selected text

         SelStart							Returns the character position of the 
         									first selected character

         SetSelection( nStart, nEnd )  Sets the selected text to be from 
                           			nStart to nEnd

-------------------------------------------------------------------------------

	Known problems:
   ---------------
   	A bug in the standard Combobox causes a selection made from th keyboard
      to be lost unless the enter key is pressed before a focus change.

-------------------------------------------------------------------------------

   ********************************************************************
   *                                                                  *
   *   Warning: This code is not warranteed in any way whatsoever.    *
   *   Use it at your own risk. Your use of this code releases Gary   *
   *   White from any liability whatsoever as regards the use,        *
   *   suitability or quality of this object.                         *
   *                                                                  *
   ********************************************************************


   Have fun with it!

      Gary

-----------------------------------------------------------------------------*/

#include windef.h
#include winuser.h
#ifndef _EXT_CB_DEFINES_
	#define _EXT_CB_DEFINES_
	#define NO_SOUND			0
	#define PC_SPEAKER		1
	#define DEFAULT_SOUND	0xFFFFFFFF
#endif

class extCombobox(oParent) of Combobox(oParent) custom
	protect LegalSounds

	// --------------- Custom Propertes --------------- //

   this.AddPrompt = true
   this.BeepOnError = DEFAULT_SOUND
   this.RestrictToList = true

   this.init()

   // --------------- Methods follow --------------- //

   function Init

   	// Prototypes the API calls used

      // Prototype SendMessage two different ways, so that we can send
      // either a numeric value (normal) or a string.  This works because
      // VdB represents the CSTRING data type as a pointer to the string.

      local bRet

      bRet = true

      try
      	this.LegalSounds = {0, 1, 0x10, 0x20, 0x30, 0x40, 0xFFFFFFFF}

         if type('SendMessageN') # 'FP'
            extern LRESULT SendMessageN(CHANDLE, UINT, WPARAM, LPARAM) user32 ;
                         from "SendMessageA"
         endif

         if type('SendMessageC') # 'FP'
            extern LRESULT SendMessageC(CHANDLE, UINT, WPARAM, LPSTR) user32 ;
                         from "SendMessageA"
         endif

         if type('ApiBeep') # 'FP'
            extern BOOL ApiBeep( DWORD, DWORD ) kernel32 from 'Beep'
         endif

         if type('MessageBeep') # 'FP'
			   extern BOOL MessageBeep(UINT) user32
         endif

      catch (exception e)
      	msgbox(e.message, 'Error '+e.code, 16)
         bRet := false
      endtry

      return bRet		// end of Init



   function SelectedText

   	// Returns the highlighted portion of the value

      local n, cRet, oCtrl

      cRet = ''

      try
         if this.style < 2

            oCtrl = form.activeControl
            this.setfocus()
            n = SendMessageN(this.hWnd, CB_GETEDITSEL, 0, 0)
            cRet := substr(this.value, LOWORD(n)+1, HIWORD(n) - LOWORD(n))
            oCtrl.setfocus()

         endif

      catch (exception e)
      	msgbox(e.message, 'Error '+e.code, 16)
      endtry

      return cRet		// end of SelectedText



   function SelStart

   	// Returns the beginning character position of the start
      // of the currently highlighted selection

      local n

      n = 0

      try
         if this.style < 2

         	n := int(LOWORD(SendMessageN(this.hWnd, CB_GETEDITSEL, 0, 0) + 1))

         endif
      catch (exception e)
      	msgbox(e.message, 'Error '+e.code, 16)
      endtry

      return n		// end of SelStart



   function SelEnd

   	// Returns the ending character position of the start
      // of the currently highlighted selection

      local n

      n = 0

      try
	      n := SendMessageN(this.hWnd, CB_GETEDITSEL, 0, 0)
      catch (exception e)
      	msgbox(e.message, 'Error '+e.code, 16)
      endtry

      return int(HIWORD(n))		// end of SelEnd



   function SelLength

   	// Returns the length of the currently highlighted selection
      // This is useful for enabling/disabling Copy and Cut menu 
      // items or buttons.

      local n

      n = 0

      try
	      n := SendMessageN(this.hWnd, CB_GETEDITSEL, 0, 0)
      catch (exception e)
      	msgbox(e.message, 'Error '+e.code, 16)
      endtry

      return int(HIWORD(n) - LOWORD(n))		// end of SelLength



   function SetSelection( nStart, nEnd )

   	// Selects (highlights) the characters from nStart to nEnd

   	local lParam, bRet

      bRet = true

      try
         nStart := max( 0, nStart-1 )
         nEnd := max(nStart, min(len(''+this.value), nEnd))
         lParam = MAKELONG(nStart, nEnd)
         SendMessageN(this.hWnd, CB_SETEDITSEL, 0, lParam)
      catch (exception e)
      	msgbox(e.message, 'Error '+e.code, 16)
         bRet := false
      endtry

      return		// end of SetSelection



   function DropDown( bShow )

   	// Displays, or hides the drop down list depending 
      // on the parameter.  If bShow is TRUE, the list is
      // displayed.  If bShow is FALSE, the list is retracted.

      local bRet

      bRet = true

      try
         SendMessageN( this.hwnd, CB_SHOWDROPDOWN, ;
            iif(bShow,1,0), 0 )
      catch (exception e)
      	msgbox(e.message, 'Error '+e.code, 16)
         bRet := false
      endtry

      return bRet		// end of DropDown



   function FindString( cString )

   	// Selects the list item cString if it is found.
      // Returns TRUE if found, or FALSE if not.

   	local bRet

      bRet = false

      try
         bRet = SendMessageC(this.hWnd, CB_FINDSTRING, 0, cString) => 0

         if bRet

            SendMessageC(this.hWnd, CB_SELECTSTRING, 0, cString)

         endif

      catch (exception e)
      	msgbox(e.message, 'Error '+e.code, 16)
      endtry

   	return bRet		// end of FindString



   function onKey(nChar, nPosition,bShift,bControl)

   	// Implements AutoCompletion:
   	// Selects the item which matches the value of the control
      // Preserves the user's typing and highlights the remainder 
      // of the text

   	local c

      c=this.value

      if nChar > 31 and nChar < 128

         if this.findstring(c)

         	this.DropDown(true)
            this.value = trim(c)
         	this.setSelection(len(c)+1, 99)

         else

         	if this.RestrictToList

               this.ErrBeep()
            	c := stuff(c, nPosition-1, 1, '')
               this.value := c
               this.keyboard(replicate('{RightArrow}',nPosition-1))
	         	this.setSelection(len(c)+1, 99)

            else

               this.DropDown(false)
               this.value = c
               this.keyboard(replicate('{RightArrow}',nPosition-1))

            endif

         endif

      endif

      return		// end of onkey



   function onLostFocus
   	this.CheckAddString()
      return



   function onChange
   	this.CheckAddString()
      return



   function CheckAddString

   	// If the datasource is a field and the value entered by the
      // user is not found, prompts to see if the user wants to add
      // the entered value.

   	local x, o

      if this.AddPrompt and (not this.RestrictToList)

         if type('this.CheckAdd') == 'U'

            this.CheckAdd = ''

         endif

         if this.CheckAdd # upper(trim(''+this.value)) and ;
               type('this.datasource') == 'O' and ;
               not this.findString(this.value)

            this.CheckAdd = upper(trim(''+this.value))

            if msgbox("The value you entered was not in the list." + chr(13) +;
                     "Do you want to add the entered value to the list?",;
                     "Value not found", 32 + 4) == 6

               x = this.value

               try
               	// this.datasource = field
                  // ...parent = fields array
                  // ......parent = rowset
                  this.datasource.parent.parent.beginappend()

                  this.datasource.value := this.value

                  this.datasource.parent.parent.save()

                  msgbox('The value was successfully added','Value added', 64)

               catch(exception e)
                  msgbox(e.message,'Error',16)
               endtry

               this.datasource := this.datasource
               this.findString(''+this.datasource.value)

            endif
         endif
      endif

      return		// end of CheckAddString



   function ErrBeep

		// Uses the Windows API to sound a beep through the PC speaker
      // or to play the selected system sound through the sound card.

      if this.LegalSounds.scan(this.BeepOnError) > 0

         do case

            case this.BeepOnError = NO_SOUND
               // Do Nothing

            case this.BeepOnError = PC_SPEAKER
               ApiBeep( 512, 100)

            otherwise
               MessageBeep(this.BeepOnError)

         endcase

      endif

      return		// end of ErrBeep

endclass

