   /*
   --------------------------------------------------------
   Filename....: filestamp.cc
   ClassName...: FileStamp
   Purpose.....: object to get and set file date/time attributes.
   Programmer..: Rick Miller with help from Gary White.
   Date........: May 31, 2002
   Notes.......: None.
   Written for.: dB2K 0.1 and newer.
   Rev. History: March 28, 2005  -  added getTimeSeparator for
                 compatibility as suggested by Simone Bartoccioni.
                 February 10, 2005  -  renamed all external functions.
                 July 25, 2002   -  revision to method setFilestamp
   Dependencies: None.
   Calls.......: See x_InitExterns method.
   Called by...: Any.
   Usage.......: set procedure to :dUFLP:filestamp.cc additive
   Example.....: See below.
   --------------------------------------------------------
   Example:
   --------------------------------------------------------
      oRef = new filestamp()
      oRef.set("date", date())
      oRef.set("time", time())
      oRef.set("filename", "c:\myapp\myapp.ini")
      oRef.set("filestamp")
      oRef.close()
      oRef := null
      close procedure :dUFLP:filestamp.cc
   --------------------------------------------------------
   Properties:          Default:
   --------------------------------------------------------
      filename <char>   space(254)  drive:\path\file or UNC\file
   --------------------------------------------------------
   Keywords for get, set methods:
   --------------------------------------------------------
      date, filename, filestamp, help, time.
   --------------------------------------------------------
   Methods:
   --------------------------------------------------------
      close()     call to release
      get(<char> keyword)
         returns object property or true/false (success/fail)
      help([<char> topic])
      inspect()   display properties in the inspector.
      release()   release object from memory
      set(<char> keyword, [value])
         returns true/false (success/fail)
   */
#define  CASTBYTE(a)    (mod(a,2^8))
#define  CASTWORD(a)    (mod(a,2^16))
#define  MAKEWORD(a, b) (bitor(CASTBYTE(a),bitlshift(CASTBYTE(b), 8)))
#define  LOBYTE(w)      (CASTBYTE(w))
#define  HIBYTE(w)      (bitrshift(CASTWORD(w),8))
#define  CR             chr(13)
#define  FILESTAMP_PUBLISHERINFO "Rick Miller"
#define  FILESTAMP_VERSIONINFO   "5.0329"
//---------------------------------------------------------------//
//          start of filestamp constructor.
//---------------------------------------------------------------//
class filestamp
   this.className =  "FileStamp"
   this.fileName  =  space(254)

   // protected methods.
   protect  checkMessage, getDate, getFilestamp,;
            getTime, getTimeSeparator, setDate, setDateIn,;
            setFilestamp, setFiletimeFromFilestamp,;
            setFiletimeFromSystime, setFiletimeFromThis,;
            setSystimeFromFiletime,;
            setSystimeFromThis, setThisFromFiletime,;
            setThisFromSystime, setTime, setTimeIn, update,;
            x_InitExterns
   // protected properties.
   protect  bokay, date, fileTime, hour, message,  minute,;
            month, runtime, second, sysTime, year

   this.second    =  0
   this.minute    =  0
   this.hour      =  0
   this.date      =  1
   this.month     =  1
   this.year      =  2002
   this.sysTime   =  space(8)
   this.fileTime  =  space(4)
   this.bokay     =  false
   this.message   =  space(4)
   this.runtime   =  iif("run" $ lower(version(0)), true, false)
   this.versioninfo  =  FILESTAMP_VERSIONINFO
   this.publishinfo  =  FILESTAMP_PUBLISHERINFO

   class::x_InitExterns()
   //------------------------------------------------------------//
   //       end of constructor   -  methods below.
   //------------------------------------------------------------//
   function checkMessage
      if not this.bokay and not this.runtime
         msgbox(this.message, [Error: filestamp.cc !!], 48)
      endif
   return true

   //------------------------------------------------------------//
   function close
      this.release()
   return

   //------------------------------------------------------------//
   function get(p1)
      Local cRet, c1
      cRet  = ""
      c1 =  lower("" + p1)
      if c1 == "date"
         cRet  := this.getDate()
      elseif c1 == "filename"
         cRet  := (this.fileName)
      elseif c1 == "filestamp"
         cRet  := this.getFilestamp()
      elseif c1 == "help"
         cRet  := this.help("intro")
      elseif c1 == "time"
         cRet  := this.getTime()
      else
         this.message   := ;
            "Unable to Get( " + c1 + " ) " +;
            CR + CR + "Invalid parameter"
         this.bokay  := false
         this.bokay  := this.checkMessage()
      endif
   return(cRet)
   // returns property value or ""

   //------------------------------------------------------------//
   function getDate
      Local d
      private c
      c  =  set("date")
      set date to MDY
      d  =  ctod(str(this.month,2, 0, "0") +;
            "\" +;
            str(this.date, 2, 0, "0") +;
            "\" +;
            str(this.year, 4, 0, "0"))
      set date to &c
   return(d)
   // returns <date exp>.

   //------------------------------------------------------------//
   function getFilestamp
      this.bokay  := false
      if not empty(this.fileName)
         if file(this.fileName)
            if this.setFiletimeFromFilestamp()
               this.bokay  := this.setThisFromFiletime()
            endif
         else
            this.message   := ;
               [The filename ] + CR + CR +;
              this.fileName + CR + CR +;
              [ is Unable to be found]
         endif
      else
         this.message   := [A Filename has not been assigned]
      endif
      this.checkMessage()
   return(this.bokay)

   //------------------------------------------------------------//
   function getTime
      Local cSep
      cSep  =  class::getTimeSeparator()
   return   str(this.hour,   2, 0, "0") + cSep +;
            str(this.minute, 2, 0, "0") + cSep +;
            str(this.second, 2, 0, "0")   //  returns <time exp>

   //------------------------------------------------------------//
   function getTimeSeparator
      Local cRet, cTime, i, nAsc
      cRet  =  ":"
      cTime =  time()
      for i = 0 to cTime.length - 1
         nAsc  =  cTime.asc(cTime.substring(i, i + 1))
         if nAsc < 48 or nAsc > 57
            cRet  := chr(nAsc)
            i  := cTime.length
         endif
      endfor
   return cRet

   //------------------------------------------------------------//
   function help(p1)
      if not this.runtime
         Local cFile, cHelp
         cFile =  left(program(1), rat(".", program(1)) - 1)
         cHelp =  setto("help")
         if file(cFile + ".hlp")
            set help to (cFile)
            if type("argvector(1)") == "C"
               private cTopic
               cTopic   =  "help " + p1
               &cTopic
               release cTopic
            else
               help intro
            endif
            set help to (cHelp)
         endif
      endif
   return

   //------------------------------------------------------------//
   function inspect
      inspect(this)
   return

   //------------------------------------------------------------//
   function release
      try
         release object this
      catch(exception e)
      endtry
   return

   //------------------------------------------------------------//
   function set(p1, p2)
      Local bRet, c1
      bRet  =  false
      c1 =  lower("" + p1)
      if c1 == "date"
         bRet  := this.setDate(p2)
      elseif c1 == "filename"
         if type("argvector(2)") == "C"
            if file(p2)
               this.fileName  := (p2)
               bRet := true
            else
               this.fileName  := ""
            endif
         endif
      elseif c1 == "filestamp"
         bRet :=  this.setFilestamp()
      elseif c1 == "help"
         bRet :=  this.help(p2)
      elseif c1 == "time"
         bRet :=  this.setTime(p2)
      else
         this.bokay  := false
         this.message := "Unable to Set( " + c1 + " ) " +;
            CR + CR + "Invalid parameter"
         this.bokay  := this.checkMessage()
      endif
   return iif(bRet, true, false)
   // return true/false (success/fail)

   //------------------------------------------------------------//
   function setDate(d)
      this.bokay  := false
      this.message   := ;
         [You have supplied an InValid Date] +;
         CR + CR +;
         [Current format is: ] + set("date") +;
         [ ] + dtoc(date())
      if type("argvector(1)") == "D"
         //  input the parameter d
         this.bokay  := this.setDateIn(d)
      endif
      this.checkMessage()
   return(this.bokay)

   //------------------------------------------------------------//
   function setDateIn(d)
      this.year   := year(d)
      this.month  := month(d)
      this.date   := day(d)
   return true

   //------------------------------------------------------------//
   function setFilestamp
      Local e, f
      this.bokay  := false
      this.message   := ;
         [Unable to Set Date/Time for:] + CR + this.fileName
      f  =  new file()
      if f.exists(this.fileName)
         if this.update()
            // revision July 25, 2002
            // added try/catch/endtry
            try
               f.open(this.fileName, "w")
               this.bokay  := RMM_SetFileTime(f.handle,;
                  this.fileTime, this.fileTime, this.fileTime )
               f.close()
            catch(exception e)
               this.message   += CR + CR + e.message +;
                  CR + CR + "Line # " + e.lineno
               this.bokay  := false
               f.close()
            endtry
         endif
      endif
      release object f
      f  := null
      this.checkMessage()
   return(this.bokay)

   //------------------------------------------------------------//
   function setFiletimeFromFilestamp
      Local c1, f
      c1 =  space(4)
      this.bokay  := false
      f  =  new file()
      try
         f.open(this.fileName, "r")
         if RMM_GetFileTime(f.handle, null, null, c1 )
            this.fileTime  := c1
            this.bokay  := true
         else
            this.message   := ;
               [Unable to Set this.filetime from File Stamp]
         endif
      catch(exception e)
         this.message   := ;
            [Filename: ] + e.filename + CR + CR +;
            e.message + CR + CR + [Line #: ] + e.lineno
      endtry
      f.close()
      release object f
      f  := null
   return(this.bokay)

   //------------------------------------------------------------//
   function setFiletimeFromSystime
      Local c1, c2
      c1 =  space(4)
      c2 =  space(4)
      if RMM_SystemTimeToFileTime((this.sysTime), c2)
        RMM_LocalFileTimeToFileTime(c2, c1)
      endif
      if not empty(c1)
         this.fileTime  := c1
         this.bokay  := true
      else
         this.message   := ;
            [Unable to Set this.filetime from this.systime] + CR +;
            this.fileName + CR +;
            dtoc(this.dateout()) + [ ] + this.timeout()
      endif
   return(this.bokay)
   // returns true/false (success/fail).

   //------------------------------------------------------------//
   function setFiletimeFromThis
      this.bokay  := false
      if this.setSystimeFromThis()
         this.bokay  := this.setFiletimeFromSystime()
      else
         this.message   := [Unable to Set this.systime from this]
      endif
   return(this.bokay)
   // returns true/false (success/fail).

   //------------------------------------------------------------//
   function setSysTimeFromFileTime
      Local c1, c2
      c1 =  space(8)
      c2 =  space(4)
      this.bokay  := false
      RMM_FileTimeToLocalFileTime((this.fileTime), c2)
      RMM_FileTimeToSystemTime(c2, c1)
      if not empty(c1)
         this.sysTime   := c1
         this.bokay  := true
      else
         this.message   := ;
            [Unable to Set this.systime from this.filetime]
      endif
   return(this.bokay)
   // returns true/false (success/fail).

   //------------------------------------------------------------//
   function setSystimeFromThis
      //  places this.year, this.month, this.date
      //  this.hour, this.minute, this.second
      //  into this.systime
      with(this.sysTime)
         setByte(0,   LOBYTE( this.year ))
         setByte(1,   HIBYTE( this.year ))
         setByte(2,   LOBYTE( this.month ))
         setByte(3,   HIBYTE( this.month ))
         setByte(4,   LOBYTE( this.date ))
         setByte(5,   HIBYTE( this.date ))
         setByte(6,   LOBYTE( this.date ))
         setByte(7,   HIBYTE( this.date ))
         setByte(8,   LOBYTE( int(this.hour) ))
         setByte(9,   HIBYTE( int(this.hour) ))
         setByte(10,  LOBYTE( int(this.minute) ))
         setByte(11,  HIBYTE( int(this.minute) ))
         setByte(12,  LOBYTE( int(this.second) ))
         setByte(13,  HIBYTE( int(this.second) ))
         setByte(14,  0)  //  milleseconds
         setByte(15,  0)  //  milleseconds
      endwith
   return true

   //------------------------------------------------------------//
   function setThisFromFiletime
      this.bokay := false
      if this.setsystimefromfiletime()
         this.bokay := this.setThisFromSystime()
      else
         this.message := [Unable to Set this.systime]
      endif
   return(this.bokay)
   // returns true/false (success/fail).

   //------------------------------------------------------------//
   function setThisFromSystime
      //  extracts this.year, this.month, this.date
      //  this.hour, this.minute, this.second
      //  from this.systime
      this.year   := MAKEWORD( ;
         this.sysTime.getByte(0),  this.sysTime.getByte(1))
      this.month  := MAKEWORD( ;
         this.sysTime.getByte(2),  this.sysTime.getByte(3))
      this.date   := MAKEWORD( ;
         this.sysTime.getByte(6),  this.sysTime.getByte(7))
      this.hour   := MAKEWORD( ;
         this.sysTime.getByte(8),  this.sysTime.getByte(9))
      this.minute := MAKEWORD( ;
         this.sysTime.getByte(10), this.sysTime.getByte(11))
      this.second := MAKEWORD( ;
         this.sysTime.getByte(12), this.sysTime.getByte(13))
   return true

   //------------------------------------------------------------//
   function setTime(t)
      this.bokay := false
      this.message := ;
         [You have supplied an InValid Time] + CR + CR +;
         [Valid example is: ] + time()
      if type("argvector(1)") == "C"
         Local cSep
         cSep  =  class::getTimeSeparator()
         if substr(t, 3, 1) == cSep and ;
            substr(t, 6, 1) == cSep and ;
            len(t) > 7
            // input the parameter t.
            this.bokay  := this.setTimeIn(left(t, 8), cSep)
         endif
      endif
      this.checkMessage()
   return(this.bokay)
   // returns true/false (success/fail)

   //------------------------------------------------------------//
   function setTimeIn(t, cSep)
      this.hour   := int(mod(val(t), 24))
      this.minute := int(mod(val(substr(t,  at(cSep, t) + 1, 2)), 60))
      this.second := int(mod(val(substr(t, rat(cSep, t) + 1, 2)), 60))
   return true

   //------------------------------------------------------------//
   function update
      // updates this.sysTime, this.fileTime
      // from this.year, this.month, this.date,
      // this.hour, this.minute, this.second
      this.setFiletimeFromThis()
   return(this.bokay)
   // returns true/false (success/fail).

   //------------------------------------------------------------//
   function x_InitExterns
      if not type("RMM_FileTimeToLocalFileTime") == "FP"
         extern CLOGICAL RMM_FileTimeToLocalFileTime(CPTR, CPTR) ;
            kernel32 from "FileTimeToLocalFileTime"
      endif
      if not type("RMM_FileTimeToSystemTime") == "FP"
         extern CLOGICAL RMM_FileTimeToSystemTime(CPTR, CPTR) ;
            kernel32 from "FileTimeToSystemTime"
      endif
      if not type("RMM_GetFileTime") == "FP"
         extern CLOGICAL RMM_GetFileTime( ;
            CHANDLE, CPTR, CPTR, CPTR ) ;
            kernel32 from "GetFileTime"
      endif
      if not type("RMM_LocalFileTimeToFileTime") == "FP"
         extern CLOGICAL RMM_LocalFileTimeToFileTime( ;
            CPTR, CPTR) kernel32 from "LocalFileTimeToFileTime"
      endif
      if not type("RMM_SetFileTime") == "FP"
         extern CLOGICAL RMM_SetFileTime( ;
            CHANDLE, CPTR, CPTR, CPTR) kernel32 from "SetFileTime"
      endif
      if not type("RMM_SystemTimeToFileTime") == "FP"
         extern CLOGICAL RMM_SystemTimeToFileTime(CPTR, CPTR) ;
            kernel32 from "SystemTimeToFileTime"
      endif
   return

endclass
//---------------------------------------------------------------//
//          end of filestamp.
//---------------------------------------------------------------//
