/*
    --------------------------------------------------------------
    Programmer..: Peter Rorlick
    Program.....: UseExclusive
    Date........: December 7, 1999
    Notes.......: Visual dBASE 7 has an interesting quirk --
                  you can open a table with the "Exclusive"
                  clause (using XDML) and not receive an
                  error if it is in use already. The table 
                  is opened, but not exclusively.

    Written for.: Visual dBASE 7.01
    Rev. History: Jan. 25, 1999 -- first release
                  Dec. 7, 1999 -- Modifications by Bruce Beecham
                        to allow opening table with an alias
                        and to deal with test index on tables
                        > 1000 records ...
    Calls.......: None
    Usage.......: UseExclusive( <cTableName>, [<lInNewArea>], [<cAlias>] )
    Example.....: UseExclusive( "Mytable" )     // or
                  UseExclusive( "Mytable", true ) // or
                  UseExclusive( "Mytable", true, "TestIt" )
    Returns.....: logical (true if table opened exclusively (and it 
                           leaves file open), false if opened in 
                           shared mode  (and it closes the file).
    Parameters..: cTableName = name of table
                  lInNewArea = logical, opens a new work
                               area to open the table in,
                               otherwise it will open it in 
                               whatever the current one is,
                               closing any table that may be
                               open there ...
                  cAlias = the alias you want the work area 
                           known as (optional).
    --------------------------------------------------------------
*/
function UseExclusive( cTableName, lInNewArea, cAlias )
	
    local LSuccess
	 private pcTableName
    pcTableName = cTableName
    // if user wants it in a new work area:
    if lInNewArea
       select select()
    endif

	private AliasString
	AliasString = cAlias
   if type("AliasString") # "C" 
      cAlias = null 
   endif
   AliasString = iif(empty(cAlias), "", " alias " + cAlias)

    // open the table with the exclusive parameter:
    use (pcTableName) exclusive &AliasString.

    // default:
    lSuccess = false

    // next we need to see if we can actually do this:
    try
       delete tag DummyXYZ // nonexistant index tag
       // the above line will throw one of three
       // errors:
       //   110: "Operation requires exclusive use of table"
       //    53: "Tag not found: DUMMYXYZ"    or
       //    35: "Table is not indexed"
    catch( exception e )
       if e.code == 35 // message = "Error: Table is not indexed"
          try
             nRecs = reccount()
             if nRecs > 1000  // So as not to waste time.
                lOK = (msgbox("Table " + upper(pcTableName) + ;
                      " is not indexed.  There are " + nRecs + ;
                      " records.  May I generate a test index?   " +;
                      chr(13) + "If not, the test for exclusive use will fail.",;
                      "Warning of a possible delay...", 49) = 1)
             else
                lOK = true
             endif
             if lOK
                index on field(1) tag DUMMYXYZ
                delete tag DUMMYXYZ
                lSuccess = true     // Can only get here if exclusive 
                                    // is available.
             endif
          catch(exception e )
            // No code, as no success and we know the reason.
          endtry         	
       endif

       if e.code == 53 // message = "Error: Tag not found"
          // we have succeeded in opening the table exclusively
          lSuccess = true
       endif
    endtry

    // table is now open one way or another ...
    if lSuccess
       * Leave open if successful...   (but it doesn't have an alias)
    else
       use
    endif

return lSuccess
** End of program/function: UseExclusive.prg
